// avres.cpp
//
// Example program show use of the class "CmReserve".  This program
// prompts for names and test scores (between 0 and 100) until an "x"
// (exit) is typed for a name.  The scores are then printed to the
// screen along with the average score.  Both the name list and the
// score list are then written to a binary file using the reserve's
// "write" functions.  All subsequent runs of this program start by
// first reading this file using the reserve's "read" function.  The
// name list and score list are printed to the screen and then the
// user is prompted for names and scores to add to the existing list.
//
// NOTE: This program uses the CmReserve class very generically to
//       perform it's tasks.  Also see the program "avres2.cpp" which
//       uses a better and more common approach to perform the exact
//       same task on the same file.

#include <cm/include/cmstring.h>
#include <cm/include/cmreserv.h>
#include "integer.h"

const char* RESERVENAME = "AVRES.DAT";           // Reserve file name.

const int MIN = 0;                               // Minimum score value.
const int MAX = 100;                             // Maximum score value.

void printReserve();                             // Reserve output func.

CmContainer *nameRoot, *scorRoot;                // Root container ptrs.

int main()                                       // Start main.
{
  CmReserve reserve(RESERVENAME);                // Declare the reserve.

  if (reserve.exists() && reserve.read())        // Read the reserve.
  {
    nameRoot = reserve["NAMES"];                 // Get names root.
    scorRoot = reserve["SCORES"];                // Get scores root.
    printReserve();                              // Print the reserve.
  }

  else                                           // Reserve is new.
  {
    nameRoot = reserve.createRoot("NAMES");      // Create names root.
    scorRoot = reserve.createRoot("SCORES");     // Create scores root.
  }

  bool     done = false;
  CmString name;
  int      score;

  while (!done)                                  // While getting input,
  {
    cout << "Enter name (\"x\" when done): " << flush;
    cin  >> name;                                // Read name.

    if (name.length() == 1 && name[0] == 'x')    // 'x' = time to exit.
    {
      printReserve();                            // Print to screen.
      reserve.write();                           // Write to disk.
      done = true;                               // Finished.
    }

    else if (name.length() > 0)                  // Got a name.
    {
      cout << "Enter score               : " << flush;
      cin  >> score;                             // Read score.
      cin.ignore();

      if (score < MIN || score > MAX)            // Out of range.
        cout << "Score must be between " << MIN << "and " << MAX << endl;

      else
      {
        nameRoot->add(new CmString(name));       // Add name to root.
        scorRoot->add(new Integer(score));       // Add score to root.
      }
    }
  }

  return 0;
}

void printReserve()                              // Reserve print function.
{
  CmIterator *iter1 = nameRoot->newIterator();   // Dynamically create
  CmIterator *iter2 = scorRoot->newIterator();   // iterators for both lists.

  int average = 0, total = 0;
  cout << endl << "Reserve contents (name, score):" << endl;

  while (!iter1->done() && !iter2->done())       // While iterators are good,
  {                                              // Print name and score.
    Integer* pInt = (Integer*) iter2->next();
    cout << *(iter1->next()) << " - " << *pInt << endl;
    average += (int) (*pInt);
    total++;                                     // Keep a running total.
  }

  cout << "--------------------" << endl;
  cout << "Average score is " << average / total << "." << endl;

  delete iter1;                                  // Must delete the iterators.
  delete iter2;
}
