// avres2.cpp
//
// Example program show use of the class "CmReserve".  This program
// prompts for names and test scores (between 0 and 100) until an "x"
// (exit) is typed for a name.  The scores are then printed to the
// screen along with the average score.  Both the name list and the
// score list are then written to a binary file using the reserve's
// "write" functions.  All subsequent runs of this program start by
// first reading this file using the reserve's "read" function.  The
// name list and score list are printed to the screen and then the
// user is prompted for names and scores to add to the existing list.
//
// NOTE: Although they function exactly the same on the same file,
//       this program differs from "avres.cpp" in that this version
//       creates a custom reserve by deriving from the CmReserve and
//       adding application specific functionality (a better and more
//       common approach in the real world).

#include <cm/include/cmstring.h>
#include <cm/include/cmreserv.h>
#include "integer.h"

const char* RESERVENAME = "AVRES.DAT";           // Reserve file name.

const int MIN = 0;                               // Minimum score value.
const int MAX = 100;                             // Maximum score value.

class AvReserve : public CmReserve {             // Our reserve definition.
public:
  AvReserve(const char*);                        // Reserve constructor.

  void addScore(const char*, int);               // Add name/score pair.

  void print(ostream&) const;                    // Print reserve contents.
};

// "AvReserve" is our reserve's constructor.
//
AvReserve::AvReserve(const char* name)
         : CmReserve(name)                       // Construct base reserve.
{
  createRoot("NAMES");                           // Create the names root.
  createRoot("SCORES");                          // Create the scores root.
}

// "addScore" adds a name/score pair to the reserve.
//
void AvReserve::addScore(const char* name, int score)
{
  getRoot("NAMES" )->add(new CmString(name));
  getRoot("SCORES")->add(new Integer(score));
}

// "print" prints the reserve contents to the output stream.
//
void AvReserve::print(ostream& strm) const
{
  CmIterator *iter1 = getRoot("NAMES" )->newIterator(); // Get root iterators.
  CmIterator *iter2 = getRoot("SCORES")->newIterator();

  int average = 0, total = 0;
  strm << endl << "Reserve contents (name, score):" << endl;

  while (!iter1->done() && !iter2->done())       // While iterators are good,
  {                                              // Print name and score.
    Integer* pInt = (Integer*) iter2->next();
    strm << *(iter1->next()) << " - " << *pInt << endl;
    average += (int) (*pInt);
    total++;                                     // Keep a running total.
  }

  strm << "--------------------" << endl;
  strm << "Average score is " << average / total << "." << endl;

  delete iter1;                                  // Must delete the iterators.
  delete iter2;
}

// "<<" output stream operator prints the reserve to the output stream.
inline ostream& operator<<(ostream& strm, const AvReserve& reserve)
{ reserve.print(strm); return strm; }

int main()                                       // Start main.
{
  AvReserve reserve(RESERVENAME);                // Declare the reserve.

  if (reserve.exists() && reserve.read())        // If the reserve is not
    cout << reserve;                             // new, print it.

  bool     done = false;
  CmString name;
  int      score;

  while (!done)                                  // While getting input,
  {
    cout << "Enter name (\"x\" when done): " << flush; // Read name.
    cin  >> name;

    if (name.length() == 1 && name[0] == 'x')    // 'x' = time to exit.
    {
      cout << reserve;                           // Print the reserve.
      reserve.write();                           // Write the reserve.
      done = true;                               // Finished.
    }

    else if (name.length() > 0)                  // Got a name.
    {
      cout << "Enter score               : " << flush; // Read the score.
      cin  >> score;
      cin.ignore();

      if (score < MIN || score > MAX)            // Out of range.
        cout << "Score must be between " << MIN << " and " << MAX << endl;

      else
        reserve.addScore(name, score);           // Add name/score to reserve.
    }
  }
  return 0;
}
