// integer.cpp
//
// Example class implementation.  This file contains the implementations
// of the non inline functions contained in the example Integer class.
// Notice, in the first two functions, that we always use the type safe
// casting mechanism.  Since the object based containers can hold mixed
// types, this is a very good practice to get into.  Also notice the
// I/O function implementations.  These functions make this class usable
// with the C++ stream operators as well as the binary file class.

#include "integer.h"

// Macro call initializes the static type info data.
CMOBJECT_INIT(Integer, CmObject)

// Macro registers this class for binary file I/O.
CMOBJECT_REGISTER(Integer)

// "isEqual" returns true if the input object is of type "Integer"
// and if it's integer value is the same as this integer value.
//
bool Integer::isEqual(const CmObject* pObj) const
{
  Integer *pInt = CmPtrCast(Integer, pObj);

  if (pInt)
    return (_value == pInt->_value) ? true : false;

  else
    return CmObject::isEqual(pObj);
}

// "compare" returns false if the input object is not of type
// "Integer".  If it is, "compare" returns 0 if the integer values
// are equal, -1 if this integer value is less than the input, and
// 1 if this integer value is greater than the input.
//
int Integer::compare(const CmObject* pObj) const
{
  Integer *pInt = CmPtrCast(Integer, pObj);

  if (pInt)
    return ((_value == pInt->_value) ? 0 : ((_value > pInt->_value) ? 1 : -1));

  else
    return CmObject::compare(pObj);
}

// "hash" is the hash function for the integer value.  A mod is
// performed with the value and the input table size.
//
unsigned Integer::hash(unsigned size) const
{
  return (unsigned)(_value % size);
}

// "printOn" prints the integer value on the specified output
// stream.
//
void Integer::printOn(ostream& os)  const
{
  os << _value;
}

// "readFrom" reads the integer value from the specified input
// stream.
//
void Integer::readFrom(istream& is)
{
  is >> _value;
}

// "write" writes the integer value to the specified file.
//
bool Integer::write(CmIOFile& file) const
{
  return file.write(_value);
}

// "read" reads the integer value from the specified file.
//
bool Integer::read(CmIOFile& file)
{
  return file.read(_value);
}
