// integer.h
//
// Example class definition.  This simple integer class is derived
// from the abstract class "CmObject".  All of the virtual functions
// in the base class are redefined and some simple lexical operators
// are also defined making this class usable with both the object
// and template based containers.

#ifndef INTEGER_H
#define INTEGER_H

#include <cm/include/cmobject.h>

class Integer : public CmObject {          // Integer class definition.
public:
  Integer(int v = 0) : _value(v) {}        // Default integer constructor.

  operator int() const;                    // Type conversion operator.

  void value(int);                         // Set new integer value.
  int  value() const;                      // Return integer value.

  CMOBJECT_DEFINE(Integer)                 // Define base. (A MUST!!)

  bool isEqual(const CmObject*) const;     // Compare objects.
  int  compare(const CmObject*) const;     // Compare objects.

  unsigned hash(unsigned) const;           // Hash function.

  void printOn (ostream&) const;           // Print to stream.
  void readFrom(istream&);                 // Read from stream.

  bool write(CmIOFile&) const;             // Write to file.
  bool read (CmIOFile&);                   // Read from file.

  bool operator< (const Integer&) const;   // Is this <  input?
  bool operator<=(const Integer&) const;   // Is this <= input?
  bool operator> (const Integer&) const;   // Is this >  input?
  bool operator>=(const Integer&) const;   // Is this >= input?
  bool operator==(const Integer&) const;   // Is this == input?
  bool operator!=(const Integer&) const;   // Is this != input?

private:
  int _value;                              // Integer value.
};

// "int" type conversion operator returns the integer value.
inline Integer::operator int() const
{ return _value; }

// "value" sets a new integer value.
inline void Integer::value(int v)
{ _value = v; }

// "value" returns the integer value.
inline int Integer::value() const
{ return _value; }

// "<" integer less than operator.
inline bool Integer::operator<(const Integer& rInt) const
{ return (_value < rInt._value) ? true : false; }

// "<=" integer less than or equal to operator.
inline bool Integer::operator<=(const Integer& rInt) const
{ return (_value <= rInt._value) ? true : false; }

// ">" integer greater than operator.
inline bool Integer::operator>(const Integer& rInt) const
{ return (_value > rInt._value) ? true : false; }

// ">=" integer greater than or equal to operator.
inline bool Integer::operator>=(const Integer& rInt) const
{ return (_value >= rInt._value) ? true : false; }

// "==" integer equal to operator.
inline bool Integer::operator==(const Integer& rInt) const
{ return (_value == rInt._value) ? true : false; }

// "!=" integer not equal to operator.
inline bool Integer::operator!=(const Integer& rInt) const
{ return (_value != rInt._value) ? true : false; }

#endif
