// cmassoc.h
// -----------------------------------------------------------------
// Compendium - C++ Container Class Library
// Copyright (C) 1992-1995, Glenn M. Poorman, All rights reserved
// -----------------------------------------------------------------
// Association definition.
// -----------------------------------------------------------------

#ifndef _CMASSOC_H
#define _CMASSOC_H

#include <cm/include/cmobject.h>

class CmAssociation : public CmObject {              // Association definition.
public:
  CmAssociation(CmObject* = NULL, CmObject* = NULL); // Association constructor.
  CmAssociation(const CmAssociation&);               // Copy constructor.
 ~CmAssociation() {}                                 // Association destructor.

  CmAssociation& operator=(const CmAssociation&);    // Assignment operator.

  CmObject* key   () const;                          // Key access.
  CmObject* object() const;                          // Object access.
  CmObject* object(CmObject* O);                     // Set object.

  bool isEqual(const CmObject*) const;               // Compare objects.
  int  compare(const CmObject*) const;               // Compare objects.

  unsigned hash(unsigned) const;                     // Hash function.

  void printOn(ostream&) const;                      // Print on stream.

  bool write(CmIOFile&) const;                       // Write to file.
  bool read (CmIOFile&);                             // Read from file.

  CMOBJECT_DEFINE(CmAssociation)                     // Define object funcs.

private:
  CmObject *_key;                                    // Association key.
  CmObject *_object;                                 // Association object.
};

// "key" returns the key value.
inline CmObject* CmAssociation::key() const
{ return _key; }

// "object" returns the object value.
inline CmObject* CmAssociation::object() const
{ return _object; }

// "object" sets a new object value.
inline CmObject* CmAssociation::object(CmObject* O)
{ CmObject* out = _object; _object = O; return out; }

#endif
