// cmio.h
// -----------------------------------------------------------------
// Compendium - C++ Container Class Library
// Copyright (C) 1992-1995, Glenn M. Poorman, All rights reserved
// -----------------------------------------------------------------
// Object I/O management class definition.
// -----------------------------------------------------------------

#ifndef _CMIO_H
#define _CMIO_H

#include <cm/include/cmbtree.h>

class CmIO {                                    // I/O class definition.
public:

  // Functions register and unregister classes for binary file I/O.
  static void registerClass(const char*, CmCreateFunc);
  static void unRegister   (const char*);

  // Function resets the object reference table after I/O.
  static void reset();

  // Functions set/retrieve the flag telling the I/O mechanism
  // whether or not to maintain multiple references to the same
  // object when doing object I/O.
  static void maintainReferences(bool);
  static bool maintainReferences();

  // Function for reading/writing of objects to/from the file.
  static bool      writeObject(CmIOFile&, CmObject*);
  static CmObject* readObject (CmIOFile&);

private:
  static CmObject* createObject(const char*);   // Create named object.
  static bool      addToTable  (CmObject*,int); // Add object to table.
  static CmObject* getFromTable(int);           // Get object at index.
  static int       getFromTable(CmObject*);     // Get object at index.

  enum ref_flag { OBJECT=0, REFERENCE=1 };      // Reference flag.

  static CmBTree *_classes;                     // Global class registry.
  static CmBTree *_table;                       // Object reference table.
  static bool     _references;                  // Multiple ref flag.
};

// "maintainReferences" sets the multiple reference flag.
inline void CmIO::maintainReferences(bool rflag)
{ CmIO::_references = rflag; }

// "maintainReferences" returns the multiple reference flag.
inline bool CmIO::maintainReferences()
{ return CmIO::_references; }

#endif
