// cmiofile.h
// -----------------------------------------------------------------
// Compendium - C++ Container Class Library
// Copyright (C) 1992-1995, Glenn M. Poorman, All rights reserved
// -----------------------------------------------------------------
// Binary file interface definition.
// -----------------------------------------------------------------

#ifndef _CMIOFILE_H
#define _CMIOFILE_H

#include <stdio.h>
#include <cm/include/cmdefs.h>

class CmIOFile {                              // File class definition.
public:
  CmIOFile();                                 // Default constructor.
  CmIOFile(const char*);                      // File constructor.
  CmIOFile(const CmIOFile&);                  // Copy constructor.
 ~CmIOFile();                                 // File destructor.

  CmIOFile& operator=(const CmIOFile&);       // Assignment.

  bool open (const char*);                    // Open named file.
  bool close();                               // Close the file.

  operator int() const;                       // Check if file is open.
  bool isValid() const;                       // Check if file is open.

  const char* name() const;                   // Return file name.

  bool remove();                              // Close and remove file.
  bool rename(const char*);                   // Rename the file.

  size_t read (void*, size_t, size_t);        // Read buffers of size.
  bool   read (void*, size_t);                // Read one buffer of size.
  size_t write(void*, size_t, size_t);        // Write buffers of size.
  bool   write(void*, size_t);                // Write one buffer of size.

  bool read(char&);                           // Read one char.
  bool read(short&);                          // Read one short.
  bool read(int&);                            // Read one integer.
  bool read(long&);                           // Read one long.
  bool read(unsigned char&);                  // Read one unsigned char.
  bool read(unsigned short&);                 // Read one unsigned short.
  bool read(unsigned int&);                   // Read one unsigned int.
  bool read(unsigned long&);                  // Read one unsigned long.
  bool read(float&);                          // Read one float.
  bool read(double&);                         // Read one double.
  bool read(char*,   unsigned);               // Read a char array.
  bool read(short*,  unsigned);               // Read a short array.
  bool read(int*,    unsigned);               // Read an int array.
  bool read(long*,   unsigned);               // Read a long array.
  bool read(float*,  unsigned);               // Read a float array.
  bool read(double*, unsigned);               // Read a double array.

  bool write(char);                           // Write one char.
  bool write(short);                          // Write one short.
  bool write(int);                            // Write one integer.
  bool write(long);                           // Write one long.
  bool write(unsigned char);                  // Write one unsigned char.
  bool write(unsigned short);                 // Write one unsigned short.
  bool write(unsigned int);                   // Write one unsigned int.
  bool write(unsigned long);                  // Write one unsigned long.
  bool write(float);                          // Write one float.
  bool write(double);                         // Write one double.
  bool write(char*,   unsigned);              // Write a char array.
  bool write(short*,  unsigned);              // Write a short array.
  bool write(int*,    unsigned);              // Write an int array.
  bool write(long*,   unsigned);              // Write a long array.
  bool write(float*,  unsigned);              // Write a float array.
  bool write(double*, unsigned);              // Write a double array.

#if !defined(NO_STANDARD_BOOL)
  bool read (bool&);                          // Read one boolean.
  bool read (bool*, unsigned);                // Read a boolean array.
  bool write(bool);                           // Write one boolean.
  bool write(bool*, unsigned);                // Write a boolean array.
#endif

  bool  write(const char*);                   // Write strlen and string.
  char* read ();                              // Read string written by above.

  int  seek  (long, int);                     // Set file position for stream.
  long tell  ();                              // Return current file position.
  void rewind();                              // Set position to beginning.
  int  getPos(fpos_t*);                       // Get current position in ptr.
  int  setPos(const fpos_t*);                 // Set position to ptr.

  void clearError();                          // Clear all error indicators.
  bool eof       ();                          // Check for end of file.
  bool error     ();                          // Check for error.
  void printError(const char*);               // Print error message.

  static bool  exists  (const char*);         // See if named file exists.
  static bool  remove  (const char*);         // Remove named file from disk.
  static char* tempName(char*);               // Create non-existing file name.

  CmIOFile& operator>>(char&);                // Read one char.
  CmIOFile& operator>>(short&);               // Read one short.
  CmIOFile& operator>>(int&);                 // Read one integer.
  CmIOFile& operator>>(long&);                // Read one long.
  CmIOFile& operator>>(unsigned char&);       // Read one unsigned char.
  CmIOFile& operator>>(unsigned short&);      // Read one unsigned short.
  CmIOFile& operator>>(unsigned int&);        // Read one unsigned int.
  CmIOFile& operator>>(unsigned long&);       // Read one unsigned long.
  CmIOFile& operator>>(float&);               // Read one float.
  CmIOFile& operator>>(double&);              // Read one double.
  CmIOFile& operator>>(char*&);               // Read strlen and string.

  CmIOFile& operator<<(char);                 // Write one char.
  CmIOFile& operator<<(short);                // Write one short.
  CmIOFile& operator<<(int);                  // Write one integer.
  CmIOFile& operator<<(long);                 // Write one long.
  CmIOFile& operator<<(unsigned char);        // Write one unsigned char.
  CmIOFile& operator<<(unsigned short);       // Write one unsigned short.
  CmIOFile& operator<<(unsigned int);         // Write one unsigned int.
  CmIOFile& operator<<(unsigned long);        // Write one unsigned long.
  CmIOFile& operator<<(float);                // Write one float.
  CmIOFile& operator<<(double);               // Write one double.
  CmIOFile& operator<<(const char*);          // Write strlen and string.

#if !defined(NO_STANDARD_BOOL)
  CmIOFile& operator>>(bool&);                // Read one boolean.
  CmIOFile& operator<<(bool);                 // Write one boolean.
#endif

private:
  char *_name;                                // File name.
  FILE *_file;                                // File stream pointer.
};

// "isValid" checks if the file was successfully opened.
inline bool CmIOFile::isValid() const
{ return (_file) ? true : false; }

// "int" operator checks if the file is currently open.
inline CmIOFile::operator int() const
{ return isValid(); }

// "name" returns the file name.
inline const char* CmIOFile::name() const
{ return _name; }

// ">>" operator reads one character.
inline CmIOFile& CmIOFile::operator>>(char& c)
{ read(c); return *this; }

// ">>" operator reads one short.
inline CmIOFile& CmIOFile::operator>>(short& s)
{ read(s); return *this; }

// ">>" operator reads one integer.
inline CmIOFile& CmIOFile::operator>>(int& i)
{ read(i); return *this; }

// ">>" operator reads one long.
inline CmIOFile& CmIOFile::operator>>(long& l)
{ read(l); return *this; }

// ">>" operator reads one unsigned character.
inline CmIOFile& CmIOFile::operator>>(unsigned char& uc)
{ read(uc); return *this; }

// ">>" operator reads one unsigned short.
inline CmIOFile& CmIOFile::operator>>(unsigned short& us)
{ read(us); return *this; }

// ">>" operator reads one unsigned integer.
inline CmIOFile& CmIOFile::operator>>(unsigned int& ui)
{ read(ui); return *this; }

// ">>" operator reads one unsigned long.
inline CmIOFile& CmIOFile::operator>>(unsigned long& ul)
{ read(ul); return *this; }

// ">>" operator reads one float.
inline CmIOFile& CmIOFile::operator>>(float& f)
{ read(f); return *this; }

// ">>" operator reads one long.
inline CmIOFile& CmIOFile::operator>>(double& d)
{ read(d); return *this; }

// ">>" operator reads string length and string text.
inline CmIOFile& CmIOFile::operator>>(char*& s)
{ s = read(); return *this; }

// "<<" operator writes one character.
inline CmIOFile& CmIOFile::operator<<(char c)
{ write(c); return *this; }

// "<<" operator writes one short.
inline CmIOFile& CmIOFile::operator<<(short s)
{ write(s); return *this; }

// "<<" operator writes one integer.
inline CmIOFile& CmIOFile::operator<<(int i)
{ write(i); return *this; }

// "<<" operator writes one long.
inline CmIOFile& CmIOFile::operator<<(long l)
{ write(l); return *this; }

// "<<" operator writes one unsigned character.
inline CmIOFile& CmIOFile::operator<<(unsigned char uc)
{ write(uc); return *this; }

// "<<" operator writes one unsigned short.
inline CmIOFile& CmIOFile::operator<<(unsigned short us)
{ write(us); return *this; }

// "<<" operator writes one unsigned integer.
inline CmIOFile& CmIOFile::operator<<(unsigned int ui)
{ write(ui); return *this; }

// "<<" operator writes one unsigned long.
inline CmIOFile& CmIOFile::operator<<(unsigned long ul)
{ write(ul); return *this; }

// "<<" operator writes one unsigned float.
inline CmIOFile& CmIOFile::operator<<(float f)
{ write(f); return *this; }

// "<<" operator writes one unsigned double.
inline CmIOFile& CmIOFile::operator<<(double d)
{ write(d); return *this; }

// "<<" outputs the string length and string text.
inline CmIOFile& CmIOFile::operator<<(const char* s)
{ write(s); return *this; }

#if !defined(NO_STANDARD_BOOL)
// ">>" operator reads one boolean.
inline CmIOFile& CmIOFile::operator>>(bool& b)
{ read(b); return *this; }

// "<<" operator writes one boolean.
inline CmIOFile& CmIOFile::operator<<(bool b)
{ write(b); return *this; }
#endif

#endif
