// cmtarray.h
// -----------------------------------------------------------------
// Compendium - C++ Container Class Library
// Copyright (C) 1992-1995, Glenn M. Poorman, All rights reserved
// -----------------------------------------------------------------
// Array template definition.
// -----------------------------------------------------------------

#ifndef _CMTARRAY_H
#define _CMTARRAY_H

#include <stdlib.h>
#include <cm/include/cmtcont.h>

template <class T> class CmTArrayIterator;    // Iterator class stub.

template <class T> class CmTArray : public CmTContainer<T> {
public:
  CmTArray(unsigned = 0, unsigned = 0);       // Default array constructor.
  CmTArray(const CmTArray<T>&);               // Array copy constructor.
  CmTArray(const CmTContainer<T>&);           // Construct from container.
 ~CmTArray();                                 // Array destructor.

  CmTArray<T>& operator=(const CmTArray<T>&); // Assignment operator.

  void     delta  (unsigned);                 // Set delta value.
  unsigned delta  () const;                   // Get delta value.
  unsigned total  () const;                   // Return number of items.
  bool     resize (unsigned);                 // Resize the array.
  bool     isEmpty() const;                   // Is array empty?

  bool add      (const T&);                   // Append item to array.
  bool insertAt (int, const T&);              // Insert item at index.
  bool replaceAt(int, const T&);              // Replace item at index.
  bool remove   (const T&);                   // Remove specified item.
  bool remove   (const CmTIterator<T>&);      // Remove at index.
  bool removeAt (int);                        // Remove item at index.
  void removeAll();                           // Remove all items.

  int index(const T&)              const;     // Get index of item.
  int index(const CmTIterator<T>&) const;     // Get index of iterator item.

  void quickSort();                           // Quick sort the array.

  const T& at         (int)      const;       // Get item at index.
  const T& lookup     (const T&) const;       // Look for equal item.
  bool     contains   (const T&) const;       // Item is in array?
  unsigned occurrences(const T&) const;       // How many occurrences?

  CmTIterator<T>* newIterator() const;        // Get array iterator.

private:
  static int cmpObjs(const void*, const void*); // Quick sort compare func.

  unsigned  _delta;                           // Delta value.   
  unsigned  _total;                           // Number of items.
  T        *_entries;                         // Array of "T" pointers.

  friend CmTArrayIterator<T>;                 // Iterator can access,
};

template <class T> class CmTArrayIterator : public CmTIterator<T> {
public:
  CmTArrayIterator(const CmTArray<T>& arr)    // Iterator constructor.
                  : CmTIterator<T>(arr), _index(0) {}

  bool     done    () const;                  // Check if done iterating.
  const T& next    ();                        // Return and advance.
  const T& previous();                        // Return and backup.
  const T& current () const;                  // Return current item.
  void     first   ();                        // Move to first item.
  void     last    ();                        // Move to last item.

  const CmTArray<T>& array() const;           // Return array reference.

private:
  int _index;                                 // Current array index.
};

#if defined(__TURBOC__) || defined(__xlC__) || defined(_MSC_VER) || defined(__GNUG__)
#include <cm/include/cmtarray.cc>
#endif

#endif
