// cmtbdict.cc
// -----------------------------------------------------------------
// Compendium - C++ Container Class Library
// Copyright (C) 1992-1995, Glenn M. Poorman, All rights reserved
// -----------------------------------------------------------------
// BTree dictionary template implementation.
// -----------------------------------------------------------------


// "CmTBTreeDictionary" is the default dictionary constructor.
//
template <class T, class O>
CmTBTreeDictionary<T,O>::CmTBTreeDictionary(unsigned order)
                       : CmTBTree< CmTAssociation<T, O> >(order)
{}


// "CmTBTreeDictionary" is the dictionary copy constructor.
//
template <class T, class O>
CmTBTreeDictionary<T,O>::CmTBTreeDictionary(const CmTBTreeDictionary<T,O>& dict)
                       : CmTBTree< CmTAssociation<T, O> >(dict)
{}


// "=" assignment operator copies the specified dictionary into this
// dictionary.
//
template <class T, class O> CmTBTreeDictionary<T,O>&
CmTBTreeDictionary<T,O>::operator=(const CmTBTreeDictionary<T,O>& dict)
{
  return (CmTBTreeDictionary<T,O>&)
          CmTBTree< CmTAssociation<T, O> >::operator=(dict);
}


// "addKey" adds an association with the specified key to the dictionary.
//
template <class T, class O>
bool CmTBTreeDictionary<T,O>::addKey(const T& rKey)
{
  CmTAssociation<T,O> assoc(rKey);
  return (contains(assoc)) ? false : add(assoc);
}


// "addKeyAndObject" adds an association with the specified key and the
// specified object to the dictionary.
//
template <class T, class O>
bool CmTBTreeDictionary<T,O>::addKeyAndObject(const T& rKey, const O& rObj)
{
  CmTAssociation<T,O> assoc(rKey, rObj);
  return (contains(assoc)) ? false : add(assoc);
}


// "addAssoc" adds the specified association to the dictionary.
//
template <class T, class O>
bool CmTBTreeDictionary<T,O>::addAssoc(const CmTAssociation<T,O>& rAssoc)
{
  return (contains(rAssoc)) ? false : add(rAssoc);
}


// "replace" finds the first occurrence of an association containing the
// specified key and replaces the object value.
//
template <class T, class O>
bool CmTBTreeDictionary<T,O>::replace(const T& rKey, const O& rObj)
{
  if (remove(CmTAssociation<T,O>(rKey)))
    return addKeyAndObject(rKey, rObj);
  else
    return false;
}


// "removeKey" removes the first association containing the specified
// key from the dictionary.
//
template <class T, class O>
bool CmTBTreeDictionary<T,O>::removeKey(const T& rKey)
{
  return remove(CmTAssociation<T,O>(rKey));
}


// "lookupKey" returns a copy of the key out of the first occurrence of
// an association containing the specified key.
//
template <class T, class O>
const T& CmTBTreeDictionary<T,O>::lookupKey(const T& rKey) const
{
  return lookup(CmTAssociation<T,O>(rKey)).key();
}


// "lookupObject" returns a copy of the object out of the first occurrence
// of an association containing the specified key.
//
template <class T, class O>
const O& CmTBTreeDictionary<T,O>::lookupObject(const T& rKey) const
{
  return lookup(CmTAssociation<T,O>(rKey)).object();
}


// "lookupAssoc" returns a copy of the first occurrence of an association
// containing the specified key.
//
template <class T, class O> const CmTAssociation<T,O>&
CmTBTreeDictionary<T,O>::lookupAssoc(const T& rKey) const
{
  return lookup(CmTAssociation<T,O>(rKey));
}


// "containsKey" returns true if the dictionary contains an association
// with the specified key.
//
template <class T, class O>
bool CmTBTreeDictionary<T,O>::containsKey(const T& rKey) const
{
  return contains(CmTAssociation<T,O>(rKey));
}


// "occurrencesOfKey" returns the number of associations containing
// the specified key.
//
template <class T, class O>
unsigned CmTBTreeDictionary<T,O>::occurrencesOfKey(const T& rKey) const
{
  return occurrences(CmTAssociation<T,O>(rKey));
}
