// cmtbintr.h
// -----------------------------------------------------------------
// Compendium - C++ Container Class Library
// Copyright (C) 1992-1995, Glenn M. Poorman, All rights reserved
// -----------------------------------------------------------------
// Binary tree template definition.
// -----------------------------------------------------------------

#ifndef _CMTBINTR_H
#define _CMTBINTR_H

#include <cm/include/cmtqueue.h>
#include <cm/include/cmtstack.h>

template <class T> class CmTBinaryTreeNode;     // Node class stub.
template <class T> class CmTBinaryTreeIterator; // Iterator class stub.

template <class T> class CmTBinaryTree : public CmTContainer<T> {
public:
  CmTBinaryTree();                              // Default constructor.
  CmTBinaryTree(const CmTBinaryTree<T>&);       // Copy constructor.
  CmTBinaryTree(const CmTContainer<T>&);        // Construct from container.
 ~CmTBinaryTree();                              // Tree destructor.

  CmTBinaryTree<T>& operator=(const CmTBinaryTree<T>&);  // Assignment. 

  bool add      (const T&);                     // Add item to tree.
  bool remove   (const T&);                     // Remove item from tree.
  bool remove   (const CmTIterator<T>&);        // Remove iterator item.
  bool removeAt (int);                          // Remove item at index.
  void removeAll();                             // Remove all items.

  const T& root   () const;                     // Return root tree item.
  void     balance();                           // Balance the tree.

  const T& lookup     (const T&) const;         // Find equal item in tree.
  bool     contains   (const T&) const;         // Is equal item in tree?
  unsigned occurrences(const T&) const;         // Count occurrences of item.

  CmTIterator<T>* newIterator() const;          // Get tree iterator.

private:
  bool removeNode(CmTBinaryTreeNode<T>*, CmTBinaryTreeNode<T>*); // Remove node.
  void balanceHalf(const CmTQueue<T>&, int, int);  // Balance tree half.

  static void killNode(CmTBinaryTreeNode<T>*);  // Destroy node children.

  CmTBinaryTreeNode<T> *_root;                  // Tree root node.

  friend CmTBinaryTreeIterator<T>;              // Iterator can access.
};

template <class T> class CmTBinaryTreeIterator : public CmTIterator<T> {
public:
  CmTBinaryTreeIterator(const CmTBinaryTree<T>&); // Iterator constructor.

  bool     done    () const;                    // Check if end of container.
  const T& next    ();                          // Advance and return.
  const T& previous();                          // Return and backup.
  const T& current () const;                    // Return current item.
  void     first   ();                          // Move to first item.
  void     last    ();                          // Move to last item.

  const CmTBinaryTree<T>& tree() const;         // Return tree reference.

private:
  CmTBinaryTreeNode<T>           *_node;        // Current tree node.
  CmTStack<CmTBinaryTreeNode<T>*> _stack;       // Node stack.
};

template <class T> class CmTBinaryTreeNode {    // Node definition.
private:
  CmTBinaryTreeNode(const T& D)                 // Node constructor.
                   : _left(NULL), _data(D), _right(NULL) {}

  CmTBinaryTreeNode<T> *_left;                  // Left node from this.
  T                     _data;                  // Node key (data).
  CmTBinaryTreeNode<T> *_right;                 // Right node from this.

  friend CmTBinaryTree<T>;                      // Tree class can access.
  friend CmTBinaryTreeIterator<T>;              // Iterator can access.
};

#if defined(__TURBOC__) || defined(__xlC__) || defined(_MSC_VER) || defined(__GNUG__)
#include <cm/include/cmtbintr.cc>
#endif

#endif
