// cmtbset.cc
// -----------------------------------------------------------------
// Compendium - C++ Container Class Library
// Copyright (C) 1992-1995, Glenn M. Poorman, All rights reserved
// -----------------------------------------------------------------
// BTree set template implementation.
// -----------------------------------------------------------------


// "CmTBTreeSet" is the default set constructor.
//
template <class T> CmTBTreeSet<T>::CmTBTreeSet(unsigned sz)
                                 : CmTBTree<T>(sz)
{}


// "CmTBTreeSet" is the set copy constructor.
//
template <class T> CmTBTreeSet<T>::CmTBTreeSet(const CmTBTreeSet<T>& S)
                                 : CmTBTree<T>(S)
{}


// "CmBTreeSet" constructs a set from the specified container.
//
template <class T> CmTBTreeSet<T>::CmTBTreeSet(const CmTContainer<T>& cont)
                                 : CmTBTree<T>(3)
{
  copy(cont);
}


// "=" assignment operator copies the specified set into this set.
//
template <class T>
CmTBTreeSet<T>& CmTBTreeSet<T>::operator=(const CmTBTreeSet<T>& S)
{
  return (CmTBTreeSet<T>&) CmTBTree<T>::operator=(S);
}


// "-" difference operator returns a set containing those items
// appearing in this set that have no equal in the specified set.
//
template <class T>
CmTBTreeSet<T> CmTBTreeSet<T>::operator-(const CmTBTreeSet<T>& set) const
{
  CmTBTreeSet<T>      out;
  CmTBTreeIterator<T> iter1(*this);
  bool                success = true;
  while (!iter1.done() && success)
  {
    const T& item = iter1.next();
    if (!set.lookup(item)) success = out.add(item);
  }
  return out;
}


// "&" intersection operator returns a set containing only those
// items appearing in this set and the input set.
//
template <class T>
CmTBTreeSet<T> CmTBTreeSet<T>::operator&(const CmTBTreeSet<T>& set) const
{
  CmTBTreeSet<T>      out;
  CmTBTreeIterator<T> iter(*this);
  bool                success = true;
  while (!iter.done() && success)
  {
    const T& item = iter.next();
    if (set.lookup(item)) success = out.add(item);
  }
  return out;
}


// "|" union operator returns a set containing those items
// appearing in this set, the input set, or both sets.
//
template <class T>
CmTBTreeSet<T> CmTBTreeSet<T>::operator|(const CmTBTreeSet<T>& set) const
{
  CmTBTreeSet<T>      out;
  CmTBTreeIterator<T> iter1(*this);
  while (!iter1.done()) out.add(iter1.next());

  CmTBTreeIterator<T> iter2(set);
  while (!iter2.done()) out.add(iter2.next());
  return out;
}


// "add" adds a new item to this set making sure not to duplicate an
// existing item.
//
template <class T> bool CmTBTreeSet<T>::add(const T& rObj)
{
  return (contains(rObj)) ? false : CmTBTree<T>::add(rObj);
}
