// cmtbtree.h
// -----------------------------------------------------------------
// Compendium - C++ Container Class Library
// Copyright (C) 1992-1995, Glenn M. Poorman, All rights reserved
// -----------------------------------------------------------------
// BTree template definition.
// -----------------------------------------------------------------

#ifndef _CMTBTREE_H
#define _CMTBTREE_H

#include <cm/include/cmtcont.h>

template <class T> class CmTBTreeNode;          // Tree node class stub.
template <class T> class CmTBTreeIterator;      // Tree iterator class stub.

template <class T> class CmTBTree : public CmTContainer<T> {
public:
  CmTBTree(unsigned = 3);                       // Default tree constructor.
  CmTBTree(const CmTBTree<T>&);                 // Tree copy constructor.
  CmTBTree(const CmTContainer<T>&);             // Construct from container.
 ~CmTBTree();                                   // Tree destructor.

  CmTBTree<T>& operator=(const CmTBTree<T>&);   // Assignment operator.

  unsigned order () const;                      // Return tree order.
  unsigned height() const;                      // Return tree height.

  bool add      (const T&);                     // Add item to tree.
  bool remove   (const T&);                     // Remove item from tree.
  bool remove   (const CmTIterator<T>&);        // Remove iterator item.
  bool removeAt (int);                          // Remove at index.
  void removeAll();                             // Remove all items.

  const T& lookup     (const T&) const;         // Find equal item in tree.
  bool     contains   (const T&) const;         // See if tree contains equal.
  unsigned occurrences(const T&) const;         // Count occurrences of equal.

  CmTIterator<T>* newIterator() const;          // Create tree iterator.

private:
  CmTBTreeNode<T>* nodeSearch(const T&) const; // Search for insertion node.
  CmTBTreeNode<T>* lookupNode(const T&) const; // Get node where item is.

  bool remove(CmTBTreeNode<T>*, int);          // Remove at index in node.

  static void removeNodes(CmTBTreeNode<T>*);   // Remove node children.
  static unsigned minKeys(unsigned);           // Get minimum keys allowed.

  unsigned         _order;                     // Tree order.
  CmTBTreeNode<T> *_root;                      // Root tree node.

  friend CmTBTreeIterator<T>;                  // Iterator class can access.
};

template <class T> class CmTBTreeNode {        // Tree node definition.
private:
  CmTBTreeNode(CmTBTreeNode<T>*, unsigned);    // Node constructor.
 ~CmTBTreeNode();                              // Node destructor.

  int  shouldGo(const T&);                     // Where should item insert.
  int  index   (const T&);                     // Index where item is located.
  int  index   (CmTBTreeNode<T>*);             // Index where child is located.
  bool addAt   (int, const T&, CmTBTreeNode<T>*, int); // Add item at index.
  bool removeAt(int, int);                     // Remove item at index.
  void clear   (int);                          // Clear arrays.

  CmTBTreeNode<T>* split(unsigned);            // Split the node.

  unsigned          _total;                    // Number of items in node.
  CmTBTreeNode<T>  *_parent;                   // Parent node.
  T                *_data;                     // (order  ) data items.
  CmTBTreeNode<T> **_children;                 // (order+1) child nodes.

  friend CmTBTree<T>;                          // Tree class can access.
  friend CmTBTreeIterator<T>;                  // Iterator class can access.
};

template <class T> class CmTBTreeIterator : public CmTIterator<T> {
public:
  CmTBTreeIterator(const CmTBTree<T>&);        // Iterator constructor.

  bool     done    () const;                   // Check if at end.
  const T& next    ();                         // Return and advance.
  const T& previous();                         // Return and backup.
  const T& current () const;                   // Get current item.
  void     first   ();                         // Move to first item.
  void     last    ();                         // Move to last item.

  const CmTBTree<T>& tree() const;             // Return tree reference.

private:
  CmTBTreeNode<T> *_node;                      // Current tree node.
  int              _index;                     // Current node item.
};

#if defined(__TURBOC__) || defined(__xlC__) || defined(_MSC_VER) || defined(__GNUG__)
#include <cm/include/cmtbtree.cc>
#endif

#endif
