// cmtdlist.h
// -----------------------------------------------------------------
// Compendium - C++ Container Class Library
// Copyright (C) 1992-1995, Glenn M. Poorman, All rights reserved
// -----------------------------------------------------------------
// Doubly linked list template definition.
// -----------------------------------------------------------------

#ifndef _CMTDLIST_H
#define _CMTDLIST_H

#include <cm/include/cmtcont.h>

template <class T> class CmTDLink;                // List link class stub.
template <class T> class CmTDLinkedListIterator;  // List iterator class stub.

template <class T> class CmTDLinkedList : public CmTContainer<T> {
public:
  CmTDLinkedList() : _first(NULL), _last(NULL) {} // Default list constructor.
  CmTDLinkedList(const CmTDLinkedList<T>&);       // List copy constructor.
  CmTDLinkedList(const CmTContainer<T>&);         // Construct from container.
 ~CmTDLinkedList();                               // List destructor.

  CmTDLinkedList<T>& operator=(const CmTDLinkedList<T>&); // Assignment op.

  bool add      (const T&);                       // Add item to end of list.
  bool append   (const T&);                       // Add item to end of list.
  bool prepend  (const T&);                       // Add item to start of list.
  bool insertAt (int, const T&);                  // Insert item at index.
  bool replaceAt(int, const T&);                  // Replace item at index.
  bool remove   (const T&);                       // Remove equal item.
  bool remove   (const CmTIterator<T>&);          // Remove iterator item.
  bool removeAt (int);                            // Remove at index.
  void removeAll();                               // Remove all items from list.

  T removeFirst();                                // Remove first item.
  T removeLast ();                                // Remove last item.

  bool insertAfter (const T&, const T&);              // Insert after another.
  bool insertAfter (const CmTIterator<T>&, const T&); // Insert after iter item.
  bool insertBefore(const T&, const T&);              // Insert before another.
  bool insertBefore(const CmTIterator<T>&, const T&); // Insert before itr item.

  const T& first() const;                         // Get first item in list.
  const T& last () const;                         // Get last item in list.

  int index(const T&)              const;         // Get index of equal item.
  int index(const CmTIterator<T>&) const;         // Get index of iter item.

  const T& at         (int)      const;           // Get item at index.
  const T& lookup     (const T&) const;           // Find equal item.
  bool     contains   (const T&) const;           // See if list contains.
  unsigned occurrences(const T&) const;           // Count occurrences of item.

  CmTIterator<T>* newIterator() const;            // Create and return iterator.

private:
  CmTDLink<T> *_first;                            // First item in list.
  CmTDLink<T> *_last;                             // Last item in list.

  friend CmTDLinkedListIterator<T>;               // Iterator can access.
};

                                                  // Iterator definition.
template <class T> class CmTDLinkedListIterator : public CmTIterator<T> {
public:
  CmTDLinkedListIterator(const CmTDLinkedList<T>&); // Iterator constructor.

  bool     done    () const;                      // Check if end of list.
  const T& next    ();                            // Return and advance.
  const T& previous();                            // Return and backup.
  const T& current () const;                      // Return current.
  void     first   ();                            // Move to first item.
  void     last    ();                            // Move to last item.

  const CmTDLinkedList<T>& list() const;          // Return list reference.

private:
  CmTDLink<T> *_link;                             // Current list link.
};

template <class T> class CmTDLink {               // List link definition.
private:
  CmTDLink(const T& D) : _next(NULL), _data(D) {} // Construct link.

  CmTDLink<T> *_next;                             // Next item in list.
  T            _data;                             // List data item.
  CmTDLink<T> *_prev;                             // Previous item in list.

  friend CmTDLinkedList<T>;                       // List class can access.
  friend CmTDLinkedListIterator<T>;               // Iterator class can access.
};

#if defined(__TURBOC__) || defined(__xlC__) || defined(_MSC_VER) || defined(__GNUG__)
#include <cm/include/cmtdlist.cc>
#endif

#endif
