// cmthset.cc
// -----------------------------------------------------------------
// Compendium - C++ Container Class Library
// Copyright (C) 1992-1995, Glenn M. Poorman, All rights reserved
// -----------------------------------------------------------------
// Hash table set template implementation.
// -----------------------------------------------------------------


// "CmTHashSet" is the default set constructor.
//
template <class T> CmTHashSet<T>::CmTHashSet(unsigned sz)
                                : CmTHashTable<T>(sz)
{}


// "CmTHashSet" constructs a set which will use the specified function
// for hashing.
//
template <class T>
CmTHashSet<T>::CmTHashSet(CmTHashFunc func, unsigned sz)
             : CmTHashTable<T>(func, sz)
{}


// "CmTHashSet" is the set copy constructor.
//
template <class T> CmTHashSet<T>::CmTHashSet(const CmTHashSet<T>& S)
                                : CmTHashTable<T>(S)
{}


// "CmTHashSet" constructs a set from the specified container.
//
template <class T>
CmTHashSet<T>::CmTHashSet(CmTHashFunc func, const CmTContainer<T>& cont)
             : CmHashTable<T>(func, cont.total())
{
  copy(cont);
}


// "=" assignment operator copies the specified set into this set.
//
template <class T>
CmTHashSet<T>& CmTHashSet<T>::operator=(const CmTHashSet<T>& S)
{
  return (CmTHashSet<T>&) CmTHashTable<T>::operator=(S);
}


// "-" difference operator returns a set containing those items
// appearing in this set that have no equal in the specified set.
//
template <class T>
CmTHashSet<T> CmTHashSet<T>::operator-(const CmTHashSet<T>& set) const
{
  CmTHashSet<T> out;
  out.setHashFunction(_hFunc);
  CmTHashTableIterator<T> iter1(*this);
  bool                    success = true;
  while (!iter1.done() && success)
  {
    const T& item = iter1.next();
    if (!set.lookup(item)) success = out.add(item);
  }
  return out;
}


// "&" intersection operator returns a set containing only those
// items appearing in this set and the input set.
//
template <class T>
CmTHashSet<T> CmTHashSet<T>::operator&(const CmTHashSet<T>& set) const
{
  CmTHashSet<T> out;
  out.setHashFunction(_hFunc);
  CmTHashTableIterator<T> iter(*this);
  bool                    success = true;
  while (!iter.done() && success)
  {
    const T& item = iter.next();
    if (set.lookup(item)) success = out.add(item);
  }
  return out;
}


// "|" union operator returns a set containing those items
// appearing in this set, the input set, or both sets.
//
template <class T>
CmTHashSet<T> CmTHashSet<T>::operator|(const CmTHashSet<T>& set) const
{
  CmTHashSet<T> out;
  out.setHashFunction(_hFunc);
  CmTHashTableIterator<T> iter1(*this);
  while (!iter1.done()) out.add(iter1.next());

  CmTHashTableIterator<T> iter2(set);
  while (!iter2.done()) out.add(iter2.next());
  return out;
}


// "add" adds a new item to this set making sure not to duplicate an
// existing item.
//
template <class T> bool CmTHashSet<T>::add(const T& rObj)
{
  return (contains(rObj)) ? false : CmTHashTable<T>::add(rObj);
}
