// cmtlist.h
// -----------------------------------------------------------------
// Compendium - C++ Container Class Library
// Copyright (C) 1992-1995, Glenn M. Poorman, All rights reserved
// -----------------------------------------------------------------
// Singly linked list template definition.
// -----------------------------------------------------------------

#ifndef _CMTLIST_H
#define _CMTLIST_H

#include <cm/include/cmtcont.h>

template <class T> class CmTLink;                 // List link class stub.
template <class T> class CmTLinkedListIterator;   // List iterator class stub.

template <class T> class CmTLinkedList : public CmTContainer<T> {
public:
  CmTLinkedList() : _first(NULL), _last(NULL) {}  // Default list constructor.
  CmTLinkedList(const CmTLinkedList<T>&);         // List copy constructor.
  CmTLinkedList(const CmTContainer<T>&);          // Construct from container.
 ~CmTLinkedList();                                // List destructor.

  CmTLinkedList<T>& operator=(const CmTLinkedList<T>&); // Assignment operator.

  bool add      (const T&);                       // Add item to end of list.
  bool append   (const T&);                       // Add item to end of list.
  bool prepend  (const T&);                       // Add item to start of list.
  bool insertAt (int, const T&);                  // Insert item at index.
  bool replaceAt(int, const T&);                  // Replace item at index.
  bool remove   (const T&);                       // Remove equal item.
  bool remove   (const CmTIterator<T>&);          // Remove iterator item.
  bool removeAt (int);                            // Remove at index.
  void removeAll();                               // Remove all items from list.

  T removeFirst();                                // Remove first item.
  T removeLast ();                                // Remove last item.

  bool insertAfter (const T&, const T&);              // Insert after another.
  bool insertAfter (const CmTIterator<T>&, const T&); // Insert after iter item.
  bool insertBefore(const T&, const T&);              // Insert before another.
  bool insertBefore(const CmTIterator<T>&, const T&); // Insert before itr item.

  const T& first() const;                         // Get first item in list.
  const T& last () const;                         // Get last item in list.

  int index(const T&)              const;         // Get index of equal item.
  int index(const CmTIterator<T>&) const;         // Get index of iter item.

  const T& at         (int)      const;           // Get item at index.
  const T& lookup     (const T&) const;           // Find equal item.
  bool     contains   (const T&) const;           // See if list contains.
  unsigned occurrences(const T&) const;           // Count occurrences of item.

  CmTIterator<T>* newIterator() const;            // Create and return iterator.

private:
  CmTLink<T> *_first;                             // First item in list.
  CmTLink<T> *_last;                              // Last item in list.

  friend CmTLinkedListIterator<T>;                // Iterator can access.
};

                                                  // Iterator definition.
template <class T> class CmTLinkedListIterator : public CmTIterator<T> {
public:
  CmTLinkedListIterator(const CmTLinkedList<T>&); // Iterator constructor.

  bool     done    () const;                      // Check if end of list.
  const T& next    ();                            // Return and advance.
  const T& previous();                            // Return and backup.
  const T& current () const;                      // Return current.
  void     first   ();                            // Move to first item.
  void     last    ();                            // Move to last item.

  const CmTLinkedList<T>& list() const;           // Return list reference.

private:
  CmTLink<T> *_link;                              // Current list link.
};

template <class T> class CmTLink {                // List link definition.
private:
  CmTLink(const T& D) : _next(NULL), _data(D) {}  // Construct link.

  CmTLink<T> *_next;                              // Next item in list.
  T           _data;                              // List data item.

  friend CmTLinkedList<T>;                        // List class can access.
  friend CmTLinkedListIterator<T>;                // Iterator class can access.
};

#if defined(__TURBOC__) || defined(__xlC__) || defined(_MSC_VER) || defined(__GNUG__)
#include <cm/include/cmtlist.cc>
#endif

#endif
