// cmtokens.h
// -----------------------------------------------------------------
// Compendium - C++ Container Class Library
// Copyright (C) 1992-1995, Glenn M. Poorman, All rights reserved
// -----------------------------------------------------------------
// String tokenizer definition.
// -----------------------------------------------------------------

#ifndef _CMTOKENS_H
#define _CMTOKENS_H

#include <cm/include/cmstring.h>

class CmTokens : public CmObject {          // Tokenizer class definition.
public:
  CmTokens(const char*);                    // Tokenizer constructor.
  CmTokens(const CmTokens&);                // Tokenizer copy constructor.

  CmTokens& operator=(const CmTokens&);     // Assignment operator.

  const char* next (const char* = "\n");    // Get next token.
  void        reset();                      // Reset to start of string.

  char delimiter() const;                   // Return last used delimiter.

  bool isEqual(const CmObject*) const;      // Compare tokenizer strings.
  int  compare(const CmObject*) const;      // Compare tokenizer strings.

  unsigned hash(unsigned) const;            // Hash tokenizer string.

  void printOn(ostream&) const;             // Print tokenizer string.

  CMOBJECT_DEFINE(CmTokens)                 // Define object funcs.

private:
  CmString    _string;                      // String to tokenize.
  const char *_place;                       // Place holder.
  int         _idx;                         // Place holder index.
  CmString    _token;                       // Current string token.
  char        _delimiter;                   // Last delimiter char.
};

// "delimiter" returns the last specified delimiter character.
inline char CmTokens::delimiter() const
{ return _delimiter; }

#endif
