// cmtordar.h
// -----------------------------------------------------------------
// Compendium - C++ Container Class Library
// Copyright (C) 1992-1995, Glenn M. Poorman, All rights reserved
// -----------------------------------------------------------------
// Ordered array template definition.
// -----------------------------------------------------------------

#ifndef _CMTORDAR_H
#define _CMTORDAR_H

#include <cm/include/cmtcont.h>

template <class T> class CmTOrderedArrayIterator; // Iterator class stub.

template <class T> class CmTOrderedArray : public CmTContainer<T> {
public:
  CmTOrderedArray(unsigned = 0, unsigned = 0);  // Default array constructor.
  CmTOrderedArray(const CmTOrderedArray<T>&);   // Array copy constructor.
  CmTOrderedArray(const CmTContainer<T>&);      // Construct from container.
 ~CmTOrderedArray() { delete[] _entries; }      // Array destructor.

  CmTOrderedArray<T>& operator=(const CmTOrderedArray<T>&); // Assignment.

  void     delta  (unsigned);                   // Set delta value.
  unsigned delta  () const;                     // Get delta value.
  unsigned total  () const;                     // Return number of items.
  bool     resize (unsigned);                   // Resize the array.
  bool     isEmpty() const;                     // Is array empty?

  bool add      (const T&);                     // Append item to array.
  bool remove   (const T&);                     // Remove specified item.
  bool remove   (const CmTIterator<T>&);        // Remove iterator item.
  bool removeAt (int);                          // Remove item at index.
  void removeAll();                             // Remove all items.

  int index   (const T&)              const;    // Get index of item.
  int index   (const CmTIterator<T>&) const;    // Get index of iterator item.
  int shouldGo(const T&)              const;    // Get index for potential item.

  const T& at         (int)      const;         // Get item at index.
  const T& lookup     (const T&) const;         // Look for equal item.
  bool     contains   (const T&) const;         // Item is in array?
  unsigned occurrences(const T&) const;         // How many occurrences?

  CmTIterator<T>* newIterator() const;          // Get array iterator.

private:
  unsigned  _delta;                             // Delta value.   
  unsigned  _total;                             // Number of items.
  T        *_entries;                           // Array of "T" pointers.

  friend CmTOrderedArrayIterator<T>;            // Iterator can access,
};

template <class T> class CmTOrderedArrayIterator : public CmTIterator<T> {
public:
  CmTOrderedArrayIterator(const CmTOrderedArray<T>& arr) // Constructor.
                         : CmTIterator<T>(arr), _index(0) {}

  bool     done    () const;                    // Check if done iterating.
  const T& next    ();                          // Return and advance.
  const T& previous();                          // Return and backup.
  const T& current () const;                    // Return current item.
  void     first   ();                          // Move to first item.
  void     last    ();                          // Move to last item.

  const CmTOrderedArray<T>& array() const;      // Return array reference.

private:
  int _index;                                   // Current array index.
};

#if defined(__TURBOC__) || defined(__xlC__) || defined(_MSC_VER) || defined(__GNUG__)
#include <cm/include/cmtordar.cc>
#endif

#endif
