// cmtvec.h
// -----------------------------------------------------------------
// Compendium - C++ Container Class Library
// Copyright (C) 1992-1995, Glenn M. Poorman, All rights reserved
// -----------------------------------------------------------------
// Dynamic array template definition.
// -----------------------------------------------------------------

#ifndef _CMTVEC_H
#define _CMTVEC_H

#include <cm/include/cmdefs.h>

template <class T> class CmTVector {                 // Vector definition.
public:
  CmTVector(unsigned = 0, const T* = NULL);          // Default constructor.
  CmTVector(unsigned, const T&);                     // Construct and fill.
  CmTVector(const CmTVector<T>&);                    // Copy constructor.
 ~CmTVector();                                       // List destructor.

  CmTVector<T>& operator=(const CmTVector<T>&);      // Assignment operator.

  CmTVector<T>& operator+=(const CmTVector<T>&);       // Append items to vec.
  CmTVector<T>  operator+ (const CmTVector<T>&) const; // Append items to vec.

  T&       operator[](int);                          // Set or get an object.
  const T& operator[](int) const;                    // Get an object.

  CmTVector<T> operator()(int, unsigned) const;      // Get sub vector.

  operator const T*() const;                         // Return array pointer.

  unsigned size () const;                            // Return number of objs.
  const T* array() const;                            // Return array pointer.

  CmTVector<T>& assign(const T*, unsigned = 1);      // Assign array to vector.
  CmTVector<T>& assign(const T&, unsigned = 1);      // Assign copies of item.

  CmTVector<T>& append(const T*, unsigned = 1);      // Append array to vector.
  CmTVector<T>& append(const T&, unsigned = 1);      // Append copies of item.

  CmTVector<T>& insert(int, const CmTVector<T>&);    // Insert vector at pos.
  CmTVector<T>& insert(int, const T*, unsigned = 1); // Insert array at pos.
  CmTVector<T>& insert(int, const T&, unsigned = 1); // Insert copies of item.

  CmTVector<T>& remove(int, unsigned = 1);           // Removes items.

  CmTVector<T> subVector(int, unsigned) const;       // Get sub vector.

  void clear();                                      // Clear the list.

  bool resize(unsigned);                             // Resize the list.

private:
  unsigned _size;                                    // Number of objects.
  T       *_array;                                   // Array of objects.
};

#if defined(__TURBOC__) || defined(__xlC__) || defined(_MSC_VER) || defined(__GNUG__)
#include <cm/include/cmtvec.cc>
#endif

#endif
