// cmassoc.cpp
// -----------------------------------------------------------------
// Compendium - C++ Container Class Library
// Copyright (C) 1992-1995, Glenn M. Poorman, All rights reserved
// -----------------------------------------------------------------
// Association implementation.
// -----------------------------------------------------------------

#include <cm/include/cmassoc.h>
#include <cm/include/cmio.h>

// Macro call initializes the static type info data.
CMOBJECT_INIT(CmAssociation, CmObject)

// Macro registers this class for binary file I/O.
CMOBJECT_REGISTER(CmAssociation)


// "CmAssociation" is the association constructor.
//
CmAssociation::CmAssociation(CmObject* pKey, CmObject* pObj)
             : _key(pKey), _object(pObj)
{}


// "CmAssociation" is the association copy constructor.
//
CmAssociation::CmAssociation(const CmAssociation& assoc)
             : _key   ((assoc._key)    ? assoc._key->newCopy()    : NULL),
               _object((assoc._object) ? assoc._object->newCopy() : NULL)
{}


// "=" assignment operator copies the contents of the specified
// assocation into this assocation.
//
CmAssociation& CmAssociation::operator=(const CmAssociation& assoc)
{
  if (&assoc != this)
  {
    _key    = (assoc._key)    ? assoc._key->newCopy()    : NULL;
    _object = (assoc._object) ? assoc._object->newCopy() : NULL;
  }
  return *this;
}


// "isEqual" returns whether or not the specified object is equal to
// this object.
//
bool CmAssociation::isEqual(const CmObject* pObj) const
{
  CmAssociation *pAssoc = CmPtrCast(CmAssociation, pObj);
  return (pAssoc && _key) ? _key->isEqual(pAssoc->_key) : false;
}


// "compare" compares the specified object to this object.
//
int CmAssociation::compare(const CmObject* pObj) const
{
  CmAssociation *pAssoc = CmPtrCast(CmAssociation, pObj);
  return (pAssoc && _key) ? _key->compare(pAssoc->_key) : -1;
}


// "hash" returns a hash value based on the input parameter.
//
unsigned CmAssociation::hash(unsigned m) const
{
  return (_key) ? _key->hash(m) : CmObject::hash(m);
}


// "printOn" prints this association on the specified output stream.
//
void CmAssociation::printOn(ostream& os) const
{
  if (_key && _object)
    os << "(" << *_key << ", " << *_object << ")";
}


// "write" writes the association objects to the specified file.
//
bool CmAssociation::write(CmIOFile& file) const
{
  if (!_key || !_object) return false;
  bool success = CmIO::writeObject(file, _key);
  if (success) success = CmIO::writeObject(file, _object);
  return success;
}


// "read" reads association objects from the specified file.
//
bool CmAssociation::read(CmIOFile& file)
{
  _key = _object = NULL;
  _key = CmIO::readObject(file);
  if (!_key) return false;

  _object = CmIO::readObject(file);
  if (_object) return true;

  delete _key;
  _key = NULL;
  return false;
}
