// cmbset.cpp
// -----------------------------------------------------------------
// Compendium - C++ Container Class Library
// Copyright (C) 1992-1995, Glenn M. Poorman, All rights reserved
// -----------------------------------------------------------------
// BTree set implementation.
// -----------------------------------------------------------------

#include <cm/include/cmbset.h>

// Macro call initializes the static type info data.
CMOBJECT_INIT(CmBTreeSet, CmBTree)

// Macro registers this class for binary file I/O.
CMOBJECT_REGISTER(CmBTreeSet)


// "CmBTreeSet" is the default set constructor.
//
CmBTreeSet::CmBTreeSet(unsigned order) : CmBTree(order)
{}


// "CmBTreeSet" is the set copy constructor.
//
CmBTreeSet::CmBTreeSet(const CmBTreeSet& set) : CmBTree(set)
{}


// "CmBTreeSet" constructs a set from the specified container.
//
CmBTreeSet::CmBTreeSet(const CmContainer& cont) : CmBTree(3)
{
  copy(cont);
}


// "=" assignment operator copies the contents of the specified set
// into this set.
//
CmBTreeSet& CmBTreeSet::operator=(const CmBTreeSet& set)
{
  return (CmBTreeSet&) CmBTree::operator=(set);
}


// "-" difference operator returns a set containing those objects
// appearing in this set that have no equal in the specified set.
//
CmBTreeSet CmBTreeSet::operator-(const CmBTreeSet& set) const
{
  CmBTreeSet out;
  out.ownsObjects(false);

  CmBTreeIterator iter1(*this);
  bool            success = true;
  while (!iter1.done() && success)
  {
    CmObject* pObj = iter1.next();
    if (!set.lookup(pObj)) success = out.add(pObj);
  }
  return out;
}


// "&" intersection operator returns a set containing only those
// objects appearing in this set and the input set.
//
CmBTreeSet CmBTreeSet::operator&(const CmBTreeSet& set) const
{
  CmBTreeSet out;
  out.ownsObjects(false);

  CmBTreeIterator iter(*this);
  bool            success = true;
  while (!iter.done() && success)
  {
    CmObject *pObj = iter.next();
    if (set.lookup(pObj)) success = out.add(pObj);
  }
  return out;
}


// "|" union operator returns a set containing those objects
// appearing in this set, the input set, or both sets.
//
CmBTreeSet CmBTreeSet::operator|(const CmBTreeSet& set) const
{
  CmBTreeSet out;
  out.ownsObjects(false);

  CmBTreeIterator iter1(*this);
  while (!iter1.done()) out.add(iter1.next());

  CmBTreeIterator iter2(set);
  while (!iter2.done()) out.add(iter2.next());
  return out;
}


// "add" adds a new object to the set provided it is not already
// contained in the set.
//
bool CmBTreeSet::add(CmObject* pObj)
{
  return (contains(pObj)) ? false : CmBTree::add(pObj);
}
