// cmdeque.cpp
// -----------------------------------------------------------------
// Compendium - C++ Container Class Library
// Copyright (C) 1992-1995, Glenn M. Poorman, All rights reserved
// -----------------------------------------------------------------
// Deque (double ended queue) implementation.
// -----------------------------------------------------------------

#include <cm/include/cmdeque.h>

// Macro call initializes the static type info data.
CMOBJECT_INIT(CmDeque, CmLinkedList)

// Macro registers this class for binary file I/O.
CMOBJECT_REGISTER(CmDeque)


// "CmDeque" is the default deque constructor.
//
CmDeque::CmDeque() : CmLinkedList()
{}


// "CmDeque" is the deque copy constructor.
//
CmDeque::CmDeque(const CmDeque& deque) : CmLinkedList(deque)
{}


// "=" assignment operator copies the specified deque into this deque.
//
CmDeque& CmDeque::operator=(const CmDeque& deque)
{
  return (CmDeque&) CmLinkedList::operator=(deque);
}


// "pushLeft" places the specified object into the queue left.
//
bool CmDeque::pushLeft(CmObject* pObj)
{
  return prepend(pObj);
}


// "pushRight" places the specified object into the queue right.
//
bool CmDeque::pushRight(CmObject* pObj)
{
  return append(pObj);
}


// "popLeft" pops the left object from the deque.
//
CmObject* CmDeque::popLeft()
{
  return removeFirst();
}


// "popRight" pops the right object from the deque.
//
CmObject* CmDeque::popRight()
{
  return removeLast();
}


// "peekLeft" returns a pointer to the left object in the deque leaving the
// object in place.
// 
CmObject* CmDeque::peekLeft() const
{
  return first();
}


// "peekRight" returns a pointer to the right object in the deque leaving the
// object in place.
// 
CmObject* CmDeque::peekRight() const
{
  return last();
}
