// cmhset.cpp
// -----------------------------------------------------------------
// Compendium - C++ Container Class Library
// Copyright (C) 1992-1995, Glenn M. Poorman, All rights reserved
// -----------------------------------------------------------------
// Hash table set implementation.
// -----------------------------------------------------------------

#include <cm/include/cmhset.h>

// Macro call initializes the static type info data.
CMOBJECT_INIT(CmHashSet, CmHashTable)

// Macro registers this class for binary file I/O.
CMOBJECT_REGISTER(CmHashSet)


// "CmHashSet" is the default set constructor.
//
CmHashSet::CmHashSet(unsigned sz) : CmHashTable(sz)
{}


// "CmHashSet" is the set copy constructor.
//
CmHashSet::CmHashSet(const CmHashSet& set) : CmHashTable(set)
{}


// "CmHashSet" constructs a set from the specified container.
//
CmHashSet::CmHashSet(const CmContainer& cont)
         : CmHashTable(cont.total())
{
  copy(cont);
}


// "=" assignment operator copies the contents of the specified set
// into this set.
//
CmHashSet& CmHashSet::operator=(const CmHashSet& set)
{
  return (CmHashSet&) CmHashTable::operator=(set);
}


// "-" difference operator returns a set containing those objects
// appearing in this set that have no equal in the specified set.
//
CmHashSet CmHashSet::operator-(const CmHashSet& set) const
{
  CmHashSet out;
  out.ownsObjects(false);

  CmHashTableIterator iter1(*this);
  bool                success = true;
  while (!iter1.done() && success)
  {
    CmObject* pObj = iter1.next();
    if (!set.lookup(pObj)) success = out.add(pObj);
  }
  return out;
}


// "&" intersection operator returns a set containing only those
// objects appearing in this set and the input set.
//
CmHashSet CmHashSet::operator&(const CmHashSet& set) const
{
  CmHashSet out;
  out.ownsObjects(false);

  CmHashTableIterator iter(*this);
  bool                success = true;
  while (!iter.done() && success)
  {
    CmObject *pObj = iter.next();
    if (set.lookup(pObj)) success = out.add(pObj);
  }
  return out;
}


// "|" union operator returns a set containing those objects
// appearing in this set, the input set, or both sets.
//
CmHashSet CmHashSet::operator|(const CmHashSet& set) const
{
  CmHashSet out;
  out.ownsObjects(false);

  CmHashTableIterator iter1(*this);
  while (!iter1.done()) out.add(iter1.next());

  CmHashTableIterator iter2(set);
  while (!iter2.done()) out.add(iter2.next());
  return out;
}


// "add" adds a new object to the set provided it is not already
// contained in the set.
//
bool CmHashSet::add(CmObject* pObj)
{
  return (contains(pObj)) ? false : CmHashTable::add(pObj);
}
