// cmtokens.cpp
// -----------------------------------------------------------------
// Compendium - C++ Container Class Library
// Copyright (C) 1992-1995, Glenn M. Poorman, All rights reserved
// -----------------------------------------------------------------
// String tokenizer implementation.
// -----------------------------------------------------------------

#include <cm/include/cmtokens.h>

// Macro call initializes the static type info data.
CMOBJECT_INIT(CmTokens, CmObject)


// "CmTokens" is the tokenizer constructor.
//
CmTokens::CmTokens(const char* str)
        : _string   (str),
          _place    (_string.text()),
          _idx      (0),
          _token    (0),
          _delimiter('\n')
{}


// "CmTokens" is the tokenizer copy constructor.
//
CmTokens::CmTokens(const CmTokens& tk)
        : _string   (tk._string),
          _place    (_string.text()),
          _idx      (0),
          _token    (tk._token),
          _delimiter(tk._delimiter)
{}


// "=" assignment operator copies the specified tokenizer into this
// tokenizer.
//
CmTokens& CmTokens::operator=(const CmTokens& tk)
{
  if (&tk != this)
  {
    _string    = tk._string;
    _place     = _string.text();
    _idx       = 0;
    _token     = tk._token;
    _delimiter = tk._delimiter;
  }
  return *this;
}


// "next" searches for and returns the next token in the string using
// the input char string as the delimiter.
//
const char* CmTokens::next(const char *del)
{
  if (!*_place)
  {
    _token     = "";
    _delimiter = '\0';
    return (const char*) _token;
  }

  _idx   += strspn(_place, del);
  _place += strspn(_place, del);

  if (!*_place)
  {
    _token = "";
    return (const char*) _token;
  }

  const char* s = strpbrk(_place, del);
  unsigned extent = (s) ? s - _place : strlen(_place);

  _delimiter = (s) ? s[0] : '\0';

  int start = _idx;

  _place += extent;
  _idx   += extent;

  _token = _string(start, extent);
  return (const char*) _token;
}


// "reset" resets the tokenizer to the beginning of the string.
//
void CmTokens::reset()
{
  _place = _string.text();
  _idx   = 0;
}


// "isEqual" compares the string with the input tokenizer string.
//
bool CmTokens::isEqual(const CmObject* pObj) const
{
  CmTokens *tk = CmPtrCast(CmTokens, pObj);
  return (tk) ? _string.isEqual(&(tk->_string)) : CmObject::isEqual(pObj);
}


// "compare" compares the string with the input tokenizer string.
//
int CmTokens::compare(const CmObject* pObj) const
{
  CmTokens *tk = CmPtrCast(CmTokens, pObj);
  return (tk) ? _string.compare(&(tk->_string)) : CmObject::compare(pObj);
}


// "hash" hashes on the string.
//
unsigned CmTokens::hash(unsigned m) const
{
  return _string.hash(m);
}


// "printOn" prints the string to the specified output stream.
//
void CmTokens::printOn(ostream& os) const
{
  _string.printOn(os);
}
