// testarr.cxx
// -----------------------------------------------------------------
// Compendium - C++ Container Class Library
// Copyright (C) 1992-1995, Glenn M. Poorman, All rights reserved
// -----------------------------------------------------------------
// Functions for array class testing.
// -----------------------------------------------------------------

#include "testbase.h"
#include <cm/include/cmarray.h>
#include <cm/include/cmordarr.h>
#include <cm/include/cmtarray.h>
#include <cm/include/cmtordar.h>


// "testArray" creates and tests the object based array class.
//
void testArray()
{
  cout << "Container type: CmArray" << endl;

  CmArray array(10, 10);
  testContainer(array);

  assert(array.size() == 10);
  assert(array.total() == 0);

  assert(array.add(new CmString("aa")));
  assert(array.add(new CmString("bb")));
  assert(array.add(new CmString("cc")));
  assert(array.add(new CmString("dd")));
  assert(array.add(new CmString("ee")));
  assert(array.add(new CmString("ff")));
  assert(array.add(new CmString("gg")));
  assert(array.add(new CmString("hh")));
  assert(array.add(new CmString("ii")));
  assert(array.add(new CmString("jj")));
  assert(array.add(new CmString("kk")));
  assert(array.add(new CmString("ll")));

  assert(array.size() == 20);
  assert(array.total() == 12);

  CmString temp("mm");
  assert(array.insertAt(3, new CmString(temp)));
  array.quickSort();
  assert(array.total() == 13);
  assert(array.index(&temp) == 12);

  CmString temp2("ee");
  assert(array.replaceAt(4, new CmString(temp)));
  assert(array.index(&temp2) < 0);
  assert(array.occurrences(&temp) == 2);
}


// "testOArray" creates and tests the object based ordered array class.
//
void testOArray()
{
  cout << "Container type: CmOrderedArray" << endl;

  CmOrderedArray array(10, 10);
  testContainer(array);

  assert(array.size() == 10);
  assert(array.total() == 0);

  assert(array.add(new CmString("aa")));
  assert(array.add(new CmString("bb")));
  assert(array.add(new CmString("cc")));
  assert(array.add(new CmString("dd")));
  assert(array.add(new CmString("ee")));
  assert(array.add(new CmString("gg")));
  assert(array.add(new CmString("hh")));
  assert(array.add(new CmString("ii")));
  assert(array.add(new CmString("jj")));
  assert(array.add(new CmString("kk")));
  assert(array.add(new CmString("ll")));
  assert(array.add(new CmString("mm")));

  assert(array.size() == 20);
  assert(array.total() == 12);

  CmString temp("ff");
  assert(array.add(new CmString(temp)));
  assert(array.total() == 13);
  temp = "mm";
  assert(array.index(&temp) == 12);
}


// "testTArray" creates and tests the template based array class.
//
void testTArray()
{
  cout << "Template type: CmTArray" << endl;

  CmTArray<CmString> array(10, 10);
  testTemplate(array);

  assert(array.size() == 10);
  assert(array.total() == 0);

  assert(array.add("aa"));
  assert(array.add("bb"));
  assert(array.add("cc"));
  assert(array.add("dd"));
  assert(array.add("ee"));
  assert(array.add("ff"));
  assert(array.add("gg"));
  assert(array.add("hh"));
  assert(array.add("ii"));
  assert(array.add("jj"));
  assert(array.add("kk"));
  assert(array.add("ll"));

  assert(array.size() == 20);
  assert(array.total() == 12);

  CmString temp("mm");
  assert(array.insertAt(3, temp));
  array.quickSort();
  assert(array.total() == 13);
  assert(array.index(temp) == 12);

  CmString temp2("ee");
  assert(array.replaceAt(4, temp));
  assert(array.index(temp2) < 0);
  assert(array.occurrences(temp) == 2);
}


// "testTOArray" creates and tests the template based ordered array class.
//
void testTOArray()
{
  cout << "Template type: CmTOrderedArray" << endl;

  CmTOrderedArray<CmString> array(10, 10);
  testTemplate(array);

  assert(array.size() == 10);
  assert(array.total() == 0);

  assert(array.add("aa"));
  assert(array.add("bb"));
  assert(array.add("cc"));
  assert(array.add("dd"));
  assert(array.add("ee"));
  assert(array.add("gg"));
  assert(array.add("hh"));
  assert(array.add("ii"));
  assert(array.add("jj"));
  assert(array.add("kk"));
  assert(array.add("ll"));
  assert(array.add("mm"));

  assert(array.size() == 20);
  assert(array.total() == 12);

  CmString temp("ff");
  assert(array.add(temp));
  assert(array.total() == 13);
  temp = "mm";
  assert(array.index(temp) == 12);
}
