// testbase.cxx
// -----------------------------------------------------------------
// Compendium - C++ Container Class Library
// Copyright (C) 1992-1995, Glenn M. Poorman, All rights reserved
// -----------------------------------------------------------------
// Functions for base container class testing.
// -----------------------------------------------------------------

#include "testbase.h"


// "hashString" is a generic string hash function used for hash
// template testing.
//
unsigned hashString(const CmString& str, unsigned tsize)
{
  return str.hash(tsize);
}


// "testContainer" tests the base functionality of any object based
// container class.
//
void testContainer(CmContainer& cont)
{
  assert(cont.add(new CmString("aa")));
  assert(cont.add(new CmString("bb")));
  assert(cont.add(new CmString("cc")));
  assert(cont.add(new CmString("dd")));
  assert(cont.add(new CmString("ee")));
  assert(cont.add(new CmString("ff")));
  assert(cont.add(new CmString("gg")));
  assert(cont.add(new CmString("hh")));

  assert(cont.isEmpty() == false);
  assert(cont.total() == 8);

  CmString temp("ee");
  assert(cont.remove(&temp));
  assert(cont.total() == 7);
  assert(cont.removeAt(2));
  assert(cont.total() == 6);

  temp = "ff";
  assert(cont.contains(&temp));
  assert(cont.lookup(&temp) != NULL);
  assert(cont.occurrences(&temp) == 1);
  assert(cont.add(new CmString(temp)));
  assert(cont.occurrences(&temp) == 2);

  cont.removeAll();
  assert(cont.isEmpty());
}


// "testTemplate" tests the base functionality of any template based
// container class.
//
void testTemplate(CmTContainer<CmString>& cont)
{
  assert(cont.add("aa"));
  assert(cont.add("bb"));
  assert(cont.add("cc"));
  assert(cont.add("dd"));
  assert(cont.add("ee"));
  assert(cont.add("ff"));
  assert(cont.add("gg"));
  assert(cont.add("hh"));

  assert(cont.isEmpty() == false);
  assert(cont.total() == 8);

  CmString temp("ee");
  assert(cont.remove(temp));
  assert(cont.total() == 7);
  assert(cont.removeAt(2));
  assert(cont.total() == 6);

  temp = "ff";
  assert(cont.contains(temp));
  assert(cont.occurrences(temp) == 1);
  assert(cont.add(temp));
  assert(cont.occurrences(temp) == 2);

  cont.removeAll();
  assert(cont.isEmpty());
}
