// testdict.cxx
// -----------------------------------------------------------------
// Compendium - C++ Container Class Library
// Copyright (C) 1992-1995, Glenn M. Poorman, All rights reserved
// -----------------------------------------------------------------
// Functions for dictionary class testing.
// -----------------------------------------------------------------

#include "testbase.h"
#include <cm/include/cmdict.h>
#include <cm/include/cmbdict.h>
#include <cm/include/cmhdict.h>
#include <cm/include/cmtdict.h>
#include <cm/include/cmtbdict.h>
#include <cm/include/cmthdict.h>


// "hashAssoc" hashes associations when testing hash template.
//
unsigned hashAssoc(const CmTAssociation<CmString,CmString>& assoc, unsigned m)
{
  return assoc.key().hash(m);
}


// "testDictionary" creates and tests the object based dictionary.
//
void testDictionary()
{
  cout << "Container type: CmDictionary" << endl;

  CmDictionary dict;
  CmAssociation *assoc = new CmAssociation(new CmString("a"),
                                           new CmString("1st letter"));

  assert(dict.add(assoc));
  assert(dict.add(new CmString("b"), new CmString("2nd letter")));
  assert(dict.add(new CmString("c"), new CmString("3rd letter")));
  assert(dict.add(new CmString("d"), new CmString("4th letter")));
  assert(dict.add(new CmString("e"), new CmString("5th letter")));
  assert(dict.add(new CmString("f"), new CmString("6th letter")));

  CmString temp("c");
  assert(dict.replace(&temp, new CmString("Third letter")));
  assert(dict.lookup(&temp) != NULL);
  assert(dict.lookupObject(&temp) != NULL);
  assert(dict.lookupAssoc(&temp) != NULL);
  assert(dict.contains(&temp));
  assert(dict.occurrences(&temp) == 1);

  CmString temp2("Third letter");
  CmObject *obj = dict.lookupObject(&temp);
  assert(obj != NULL);
  assert(temp2.isEqual(obj));

  CmString *pstr1 = new CmString(temp);
  CmString *pstr2 = new CmString("Another one");
  assert(!dict.add(pstr1, pstr2));
  delete pstr1;
  delete pstr2;
}


// "testBTreeDictionary" creates and tests the object based btree dictionary.
//
void testBTreeDictionary()
{
  cout << "Container type: CmBTreeDictionary" << endl;

  CmBTreeDictionary dict;
  CmAssociation *assoc = new CmAssociation(new CmString("a"),
                                           new CmString("1st letter"));

  assert(dict.add(assoc));
  assert(dict.add(new CmString("b"), new CmString("2nd letter")));
  assert(dict.add(new CmString("c"), new CmString("3rd letter")));
  assert(dict.add(new CmString("d"), new CmString("4th letter")));
  assert(dict.add(new CmString("e"), new CmString("5th letter")));
  assert(dict.add(new CmString("f"), new CmString("6th letter")));

  CmString temp("c");
  assert(dict.replace(&temp, new CmString("Third letter")));
  assert(dict.lookup(&temp) != NULL);
  assert(dict.lookupObject(&temp) != NULL);
  assert(dict.lookupAssoc(&temp) != NULL);
  assert(dict.contains(&temp));
  assert(dict.occurrences(&temp) == 1);

  CmString temp2("Third letter");
  CmObject *obj = dict.lookupObject(&temp);
  assert(obj != NULL);
  assert(temp2.isEqual(obj));

  CmString *pstr1 = new CmString(temp);
  CmString *pstr2 = new CmString("Another one");
  assert(!dict.add(pstr1, pstr2));
  delete pstr1;
  delete pstr2;
}


// "testHashDictionary" creates and tests the object based hash dictionary.
//
void testHashDictionary()
{
  cout << "Container type: CmHashDictionary" << endl;

  CmHashDictionary dict;
  CmAssociation *assoc = new CmAssociation(new CmString("a"),
                                           new CmString("1st letter"));

  assert(dict.add(assoc));
  assert(dict.add(new CmString("b"), new CmString("2nd letter")));
  assert(dict.add(new CmString("c"), new CmString("3rd letter")));
  assert(dict.add(new CmString("d"), new CmString("4th letter")));
  assert(dict.add(new CmString("e"), new CmString("5th letter")));
  assert(dict.add(new CmString("f"), new CmString("6th letter")));

  CmString temp("c");
  assert(dict.replace(&temp, new CmString("Third letter")));
  assert(dict.lookup(&temp) != NULL);
  assert(dict.lookupObject(&temp) != NULL);
  assert(dict.lookupAssoc(&temp) != NULL);
  assert(dict.contains(&temp));
  assert(dict.occurrences(&temp) == 1);

  CmString temp2("Third letter");
  CmObject *obj = dict.lookupObject(&temp);
  assert(obj != NULL);
  assert(temp2.isEqual(obj));

  CmString *pstr1 = new CmString(temp);
  CmString *pstr2 = new CmString("Another one");
  assert(!dict.add(pstr1, pstr2));
  delete pstr1;
  delete pstr2;
}


// "testTDictionary" creates and tests the template based dictionary.
//
void testTDictionary()
{
  cout << "Template type: CmTDictionary" << endl;

  CmTDictionary<CmString,CmString> dict;
  CmTAssociation<CmString,CmString> assoc("a", "1st letter");

  assert(dict.addAssoc(assoc));
  assert(dict.addKeyAndObject("b", "2nd letter"));
  assert(dict.addKeyAndObject("c", "3rd letter"));
  assert(dict.addKeyAndObject("d", "4th letter"));
  assert(dict.addKeyAndObject("e", "5th letter"));
  assert(dict.addKeyAndObject("f", "6th letter"));

  CmString temp("c");
  assert(dict.replace(temp, "Third letter"));
  assert(dict.containsKey(temp));
  assert(dict.occurrencesOfKey(temp) == 1);

  CmString temp2("Third letter");
  CmString obj = dict.lookupObject(temp);
  assert(temp2 == obj);

  assert(!dict.addKeyAndObject(temp, "Another one"));
}


// "testTBTreeDictionary" creates and tests the template based btree dictionary.
//
void testTBTreeDictionary()
{
  cout << "Template type: CmTBTreeDictionary" << endl;

  CmTBTreeDictionary<CmString,CmString> dict;
  CmTAssociation<CmString,CmString> assoc("a", "1st letter");

  assert(dict.addAssoc(assoc));
  assert(dict.addKeyAndObject("b", "2nd letter"));
  assert(dict.addKeyAndObject("c", "3rd letter"));
  assert(dict.addKeyAndObject("d", "4th letter"));
  assert(dict.addKeyAndObject("e", "5th letter"));
  assert(dict.addKeyAndObject("f", "6th letter"));

  CmString temp("c");
  assert(dict.replace(temp, "Third letter"));
  assert(dict.containsKey(temp));
  assert(dict.occurrencesOfKey(temp) == 1);

  CmString temp2("Third letter");
  CmString obj = dict.lookupObject(temp);
  assert(temp2 == obj);

  assert(!dict.addKeyAndObject(temp, "Another one"));
}


// "testTHashDictionary" creates and tests the template based hash dictionary.
//
void testTHashDictionary()
{
  cout << "Template type: CmTHashDictionary" << endl;

  CmTHashDictionary<CmString,CmString> dict;
  CmTAssociation<CmString,CmString> assoc("a", "1st letter");

  dict.setHashFunction(hashAssoc);

  assert(dict.addAssoc(assoc));
  assert(dict.addKeyAndObject("b", "2nd letter"));
  assert(dict.addKeyAndObject("c", "3rd letter"));
  assert(dict.addKeyAndObject("d", "4th letter"));
  assert(dict.addKeyAndObject("e", "5th letter"));
  assert(dict.addKeyAndObject("f", "6th letter"));

  CmString temp("c");
  assert(dict.replace(temp, "Third letter"));
  assert(dict.containsKey(temp));
  assert(dict.occurrencesOfKey(temp) == 1);

  CmString temp2("Third letter");
  CmString obj = dict.lookupObject(temp);
  assert(temp2 == obj);

  assert(!dict.addKeyAndObject(temp, "Another one"));
}
