// testhash.cxx
// -----------------------------------------------------------------
// Compendium - C++ Container Class Library
// Copyright (C) 1992-1995, Glenn M. Poorman, All rights reserved
// -----------------------------------------------------------------
// Functions for hash table class testing.
// -----------------------------------------------------------------

#include "testbase.h"
#include <cm/include/cmhash.h>
#include <cm/include/cmthash.h>


// "testHashTable" creates and tests the object based hash table class.
//
void testHashTable()
{
  cout << "Container type: CmHashTable" << endl;

  CmHashTable table;

  assert(table.add(new CmString("aa")));
  assert(table.add(new CmString("bb")));
  assert(table.add(new CmString("cc")));
  assert(table.add(new CmString("dd")));
  assert(table.add(new CmString("ee")));
  assert(table.add(new CmString("ff")));
  assert(table.add(new CmString("gg")));
  assert(table.add(new CmString("hh")));

  assert(table.isEmpty() == false);
  assert(table.total() == 8);

  CmString temp("ee");
  assert(table.remove(&temp));
  assert(table.total() == 7);

  temp = "ff";
  assert(table.contains(&temp));
  assert(table.lookup(&temp) != NULL);
  assert(table.occurrences(&temp) == 1);
  assert(table.add(new CmString(temp)));
  assert(table.occurrences(&temp) == 2);

  table.removeAll();
  assert(table.isEmpty());
}


// "testTHashTable" creates and tests the template based hash table class.
//
void testTHashTable()
{
  cout << "Template type: CmTHashTable" << endl;

  CmTHashTable<CmString> table;
  table.setHashFunction(hashString);

  assert(table.add("aa"));
  assert(table.add("bb"));
  assert(table.add("cc"));
  assert(table.add("dd"));
  assert(table.add("ee"));
  assert(table.add("ff"));
  assert(table.add("gg"));
  assert(table.add("hh"));

  assert(table.isEmpty() == false);
  assert(table.total() == 8);

  CmString temp("ee");
  assert(table.remove(temp));
  assert(table.total() == 7);

  temp = "ff";
  assert(table.contains(temp));
  assert(table.occurrences(temp) == 1);
  assert(table.add(temp));
  assert(table.occurrences(temp) == 2);

  table.removeAll();
  assert(table.isEmpty());
}
