// testlist.cxx
// -----------------------------------------------------------------
// Compendium - C++ Container Class Library
// Copyright (C) 1992-1995, Glenn M. Poorman, All rights reserved
// -----------------------------------------------------------------
// Functions for linked list class testing.
// -----------------------------------------------------------------

#include "testbase.h"
#include <cm/include/cmlist.h>
#include <cm/include/cmtlist.h>
#include <cm/include/cmdlist.h>
#include <cm/include/cmtdlist.h>

// "testList" creates and tests the object based linked list.
//
void testList()
{
  cout << "Container type: CmLinkedList" << endl;

  CmLinkedList list;
  testContainer(list);

  assert(list.append(new CmString("bb")));
  assert(list.append(new CmString("cc")));
  assert(list.append(new CmString("dd")));
  assert(list.append(new CmString("ee")));
  assert(list.append(new CmString("ff")));
  assert(list.prepend(new CmString("aa")));

  assert(list.total() == 6);
  assert(list.size() == 6);

  CmString temp1("aa"), temp2("ff");
  CmObject *obj = list.first();
  assert(temp1.isEqual(obj));
  obj = list.last();
  assert(temp2.isEqual(obj));

  assert(list.index(&temp2) == 5);

  temp1 = "cc";
  temp2 = "compendium";
  assert(list.insertAfter(&temp1, new CmString(temp2)));
  assert(list.index(&temp2) == 3);
  temp2 = "bee";
  assert(list.insertBefore(&temp1, new CmString(temp2)));
  assert(list.index(&temp2) == 2);
  assert(list.size() == 8);
  assert(list.total() == 8);

  CmObject *front = list.removeFirst();
  CmObject *back  = list.removeLast ();
  assert(front && back);
  assert(list.size() == 6);
  assert(list.total() == 6);
  delete front;
  delete back;
}


// "testDList" creates and tests the object based doubly linked list.
//
void testDList()
{
  cout << "Container type: CmDLinkedList" << endl;

  CmDLinkedList list;
  testContainer(list);

  assert(list.append(new CmString("bb")));
  assert(list.append(new CmString("cc")));
  assert(list.append(new CmString("dd")));
  assert(list.append(new CmString("ee")));
  assert(list.append(new CmString("ff")));
  assert(list.prepend(new CmString("aa")));

  assert(list.total() == 6);
  assert(list.size() == 6);

  CmString temp1("aa"), temp2("ff");
  CmObject *obj = list.first();
  assert(temp1.isEqual(obj));
  obj = list.last();
  assert(temp2.isEqual(obj));

  assert(list.index(&temp2) == 5);

  temp1 = "cc";
  temp2 = "compendium";
  assert(list.insertAfter(&temp1, new CmString(temp2)));
  assert(list.index(&temp2) == 3);
  temp2 = "bee";
  assert(list.insertBefore(&temp1, new CmString(temp2)));
  assert(list.index(&temp2) == 2);
  assert(list.size() == 8);
  assert(list.total() == 8);

  CmObject *front = list.removeFirst();
  CmObject *back  = list.removeLast ();
  assert(front && back);
  assert(list.size() == 6);
  assert(list.total() == 6);
  delete front;
  delete back;
}


// "testTList" creates and tests the template based linked list.
//
void testTList()
{
  cout << "Template type: CmTLinkedList" << endl;

  CmTLinkedList<CmString> list;
  testTemplate(list);

  assert(list.append("bb"));
  assert(list.append("cc"));
  assert(list.append("dd"));
  assert(list.append("ee"));
  assert(list.append("ff"));
  assert(list.prepend("aa"));

  assert(list.total() == 6);
  assert(list.size() == 6);

  CmString temp1("aa"), temp2("ff");
  CmString obj = list.first();
  assert(temp1 == obj);
  obj = list.last();
  assert(temp2 == obj);

  assert(list.index(temp2) == 5);

  temp1 = "cc";
  temp2 = "compendium";
  assert(list.insertAfter(temp1, temp2));
  assert(list.index(temp2) == 3);
  temp2 = "bee";
  assert(list.insertBefore(temp1, temp2));
  assert(list.index(temp2) == 2);
  assert(list.size() == 8);
  assert(list.total() == 8);

  CmString front = list.removeFirst();
  CmString back  = list.removeLast ();
  assert(list.size() == 6);
  assert(list.total() == 6);
}


// "testTDList" creates and tests the template based doubly linked list.
//
void testTDList()
{
  cout << "Template type: CmTDLinkedList" << endl;

  CmTDLinkedList<CmString> list;
  testTemplate(list);

  assert(list.append("bb"));
  assert(list.append("cc"));
  assert(list.append("dd"));
  assert(list.append("ee"));
  assert(list.append("ff"));
  assert(list.prepend("aa"));

  assert(list.total() == 6);
  assert(list.size() == 6);

  CmString temp1("aa"), temp2("ff");
  CmString obj = list.first();
  assert(temp1 == obj);
  obj = list.last();
  assert(temp2 == obj);

  assert(list.index(temp2) == 5);

  temp1 = "cc";
  temp2 = "compendium";
  assert(list.insertAfter(temp1, temp2));
  assert(list.index(temp2) == 3);
  temp2 = "bee";
  assert(list.insertBefore(temp1, temp2));
  assert(list.index(temp2) == 2);
  assert(list.size() == 8);
  assert(list.total() == 8);

  CmString front = list.removeFirst();
  CmString back  = list.removeLast ();
  assert(list.size() == 6);
  assert(list.total() == 6);
}
