// testtree.cxx
// -----------------------------------------------------------------
// Compendium - C++ Container Class Library
// Copyright (C) 1992-1995, Glenn M. Poorman, All rights reserved
// -----------------------------------------------------------------
// Functions for tree class testing.
// -----------------------------------------------------------------

#include "testbase.h"
#include <cm/include/cmbintr.h>
#include <cm/include/cmbtree.h>
#include <cm/include/cmtbintr.h>
#include <cm/include/cmtbtree.h>


// "testBinaryTree" creates and tests the object based binary tree.
//
void testBinaryTree()
{
  cout << "Container type: CmBinaryTree" << endl;

  CmBinaryTree tree;
  testContainer(tree);

  assert(tree.add(new CmString("aa")));
  assert(tree.add(new CmString("bb")));
  assert(tree.add(new CmString("cc")));
  assert(tree.add(new CmString("dd")));
  assert(tree.add(new CmString("ee")));
  assert(tree.add(new CmString("ff")));
  assert(tree.add(new CmString("gg")));

  tree.balance();

  CmString temp("dd");
  CmObject *obj = tree.root();
  assert(temp.isEqual(obj));
}


// "testBTree" creates and tests the object based btree.
//
void testBTree()
{
  cout << "Container type: CmBTree" << endl;

  CmBTree tree;
  testContainer(tree);

  assert(tree.add(new CmString("aa")));
  assert(tree.add(new CmString("bb")));
  assert(tree.add(new CmString("cc")));
  assert(tree.add(new CmString("dd")));
  assert(tree.add(new CmString("ee")));
  assert(tree.add(new CmString("ff")));
  assert(tree.add(new CmString("gg")));

  assert(tree.order() == 3);
  assert(tree.height() == 3);
}


// "testTBinaryTree" creates and tests the template based binary tree.
//
void testTBinaryTree()
{
  cout << "Template type: CmTBinaryTree" << endl;

  CmTBinaryTree<CmString> tree;
  testTemplate(tree);

  assert(tree.add("aa"));
  assert(tree.add("bb"));
  assert(tree.add("cc"));
  assert(tree.add("dd"));
  assert(tree.add("ee"));
  assert(tree.add("ff"));
  assert(tree.add("gg"));

  tree.balance();

  CmString temp("dd");
  CmString obj = tree.root();
  assert(temp == obj);
}


// "testTBTree" creates and tests the template based btree.
//
void testTBTree()
{
  cout << "Template type: CmTBTree" << endl;

  CmTBTree<CmString> tree;
  testTemplate(tree);

  assert(tree.add("aa"));
  assert(tree.add("bb"));
  assert(tree.add("cc"));
  assert(tree.add("dd"));
  assert(tree.add("ee"));
  assert(tree.add("ff"));
  assert(tree.add("gg"));

  assert(tree.order() == 3);
  assert(tree.height() == 3);
}
